#include <AccelStepper.h>

#define motorPin1  A3      // IN1 on the ULN2003 driver
#define motorPin2  A2      // IN2 on the ULN2003 driver
#define motorPin3  A1     // IN3 on the ULN2003 driver
#define motorPin4  A0     // IN4 on the ULN2003 driver
#define MotorInterfaceType 8

AccelStepper stepper = AccelStepper(MotorInterfaceType, motorPin1, motorPin3, motorPin2, motorPin4);

// ultrasonic
#define echoPin 13
#define trigPin 12
#define maxDistance 200
int position, currentDistance;

// setup motor
#define enA 5
#define in1 10
#define in2 9
#define in3 8
#define in4 7
#define enB 6
#define MOTOR_SPEED 200

void setup() {
  
  Serial.begin(9600);
  
  //setup set motor
  // stepper.setSpeed(120);
  stepper.setMaxSpeed(5000);
  stepper.setAcceleration(500);

  //ultrasonic setup
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  currentDistance = maxDistance;

  //motor setup
  TCCR0B = TCCR0B & B11111000 | B00000010;
  pinMode(enA, OUTPUT);
  pinMode(in1, OUTPUT);
  pinMode(in2, OUTPUT);
  
  pinMode(enB, OUTPUT);
  pinMode(in3, OUTPUT);
  pinMode(in4, OUTPUT);
}

void loop() {

  float distance = calculate_distance();
  while(!distance)
    distance = calculate_distance();
 
  if(distance <= 75 && distance >= 6) {
     Serial.println(distance);
    move_forward(); 
    run_step_motor();
  }
  else {
    step_back(4000);
    float rightDistance = step_right(4000);   
    float leftDistance = step_left(6500);   
    float get_min = minim(leftDistance, rightDistance);
    if(get_min == rightDistance) {
        rotate_right();
        delay(6500);
    }
    stop_car();
    move_forward();
    delay(2000);
  }
}

void run_step_motor() {

  stepper.moveTo(stepper.currentPosition() + 200);
  stepper.runToPosition();
}

float step_left(float ms){
  rotate_left();
  delay(ms);
  stop_car();
  float leftDistance = calculate_distance();
  while(!leftDistance)
        leftDistance = calculate_distance();
  delay(ms);
  if(leftDistance < 15)
    return 200;
  return leftDistance;
}

float step_right(float ms){
    rotate_right();
    delay(ms);
    stop_car();
    float rightDistance = calculate_distance();
    while(!rightDistance)
        rightDistance = calculate_distance();
    delay(ms);
    if(rightDistance < 15)
      return 200;
    return rightDistance;
}

void step_back(float ms){
    stop_car();
    move_back();
    delay(ms);
    stop_car();
}

float minim (float a, float b) {
  if(a > b )
    return b;
  return a;
}

void stop_car() {

  digitalWrite(in1,LOW);
  digitalWrite(in2,LOW);  

  digitalWrite(in3,LOW);
  digitalWrite(in4,LOW);  

  analogWrite(enA, MOTOR_SPEED);
  analogWrite(enB, MOTOR_SPEED); 
}

void move_back() {
  digitalWrite(in1,LOW);
  digitalWrite(in2,HIGH);  

  digitalWrite(in3,LOW);
  digitalWrite(in4,HIGH);

  analogWrite(enA, MOTOR_SPEED);
  analogWrite(enB, MOTOR_SPEED); 
  // delay(500);
}

void rotate_right() {
   digitalWrite(in1,HIGH);
  digitalWrite(in2,LOW);  

  digitalWrite(in3,LOW);
  digitalWrite(in4,HIGH);

  analogWrite(enA, MOTOR_SPEED);
  analogWrite(enB, MOTOR_SPEED); 

}

void rotate_left() {
   digitalWrite(in1,LOW);
  digitalWrite(in2,HIGH);  

  digitalWrite(in3,HIGH);
  digitalWrite(in4,LOW);

  analogWrite(enA, MOTOR_SPEED);
  analogWrite(enB, MOTOR_SPEED); 
}

void move_forward() {

  digitalWrite(in1,HIGH);
  digitalWrite(in2,LOW);  

  digitalWrite(in3,HIGH);
  digitalWrite(in4,LOW);  

  analogWrite(enA, MOTOR_SPEED);
  analogWrite(enB, MOTOR_SPEED); 
}

float calculate_distance() {
  long duration;
  float distance;

  digitalWrite(trigPin, LOW);
  digitalWrite(trigPin, HIGH);
  digitalWrite(trigPin, LOW);
  
  duration = pulseIn(echoPin, HIGH);
  distance = duration * 0.034 / 2;
  return distance;
}